from datetime import date
from typing import Dict, List, Optional, Union

from belvo.enums import TaxReturnType
from belvo.resources.base import Resource


class TaxReturns(Resource):
    endpoint = "/api/tax-returns/"

    def create(
        self,
        link: str,
        year_from: str = None,
        year_to: str = None,
        *,
        attach_pdf: bool = False,
        save_data: bool = True,
        raise_exception: bool = False,
        type_: Optional[TaxReturnType] = None,
        date_from: str = None,
        date_to: str = None,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:
        """Retrieve tax returns for a link

        Retrieve yearly (annual) or monthly tax returns for a fiscal link.

        Args:
            link (str): The fiscal `link.id` you want specific tax return information for.
            year_from (str, optional): The starting year you want to get tax returns for, in `YYYY` format. Defaults to None.
            year_to (str, optional): The year you want to stop geting tax returns for, in `YYYY` format. Defaults to None.
            attach_pdf (bool, optional): When this is set to `True`, you will receive the PDF as a binary string in the response.. Defaults to `False`.
            save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.
            type_ (Optional[TaxReturnType], optional): The type of tax return to return. For yearly tax returns this must be set to `yearly`. For monthly tax returns, this field must be set to `monthly`. By default, Belvo returns the **yearly** (annual) tax returns.
            date_from (str, optional): The starting date you want to get tax returns for, in `YYYY-MM-DD` format. The value of `date_from` cannot be greater than `date_to`. Defaults to None.
            date_to (str, optional): The date you want to stop geting tax returns for, in `YYYY-MM-DD` format. The value of `date_to` cannot be greater than today's date (in other words, no future dates). Defaults to None.

        Returns:
            Dict: The details of the object. For more information on the response from the API, see our [Tax returns API documentation](https://developers.belvo.com/reference/retrievetaxreturns).
        """

        type_ = type_ if type_ else TaxReturnType.YEARLY

        data = {"link": link, "attach_pdf": attach_pdf, "save_data": save_data, "type": type_.value}

        if data["type"] == "yearly":
            year_to = year_to or str(date.today().year)
            data.update(year_to=year_to, year_from=year_from)
        else:
            data.update(date_to=date_to, date_from=date_from)

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )

    def resume(
        self,
        session: str,
        token: str,
        *,
        link: str = None,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Dict:
        raise NotImplementedError()



from typing import Dict, List, Union

from belvo.resources.base import Resource


class Owners(Resource):
    endpoint = "/api/owners/"

    def create(
        self,
        link: str,
        *,
        token: str = None,
        save_data: bool = True,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:
        """Retrieve owners for a Link

        Retrieve owner information for a specific link.

        Example:

            ```python
            # Fetch owners for a Link
            owners = client.Owners.create("b91835f5-6f83-4d9b-a0ad-a5a249f18b7c")

            # Fetch owners for a Link with and timeout after 15 seconds
            owners = client.Owners.create(
                "b91835f5-6f83-4d9b-a0ad-a5a249f18b7c",
                timeout=15
            )

        ```

        Args:
            link (str): The `link.id` that you want to get information for (UUID).
            token (str, optional): The MFA token generated by the bank in order to access the institution. Defaults to None.
            save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.

        Returns:
            Dict: The details of the object. For more information on the response from the API, see our [Owners API documentation](https://developers.belvo.com/reference/retrieveowners).
        """

        data = {"link": link, "save_data": save_data}

        if token:
            data.update(token=token)

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )



from belvo.investments.portfolios import InvestmentsPortfolios  # noqa
from belvo.investments.transactions import InvestmentsTransactions  # noqa
from belvo.resources.accounts import Accounts  # noqa
from belvo.resources.balances import Balances  # noqa
from belvo.resources.incomes import Incomes  # noqa
from belvo.resources.institutions import Institutions  # noqa
from belvo.resources.invoices import Invoices  # noqa
from belvo.resources.links import Links  # noqa
from belvo.resources.owners import Owners  # noqa
from belvo.resources.recurring_expenses import RecurringExpenses  # noqa
from belvo.resources.risk_insights import RiskInsights  # noqa
from belvo.resources.statements import Statements  # noqa
from belvo.resources.tax_compliance_status import TaxComplianceStatus  # noqa
from belvo.resources.tax_returns import TaxReturns  # noqa
from belvo.resources.tax_status import TaxStatus  # noqa
from belvo.resources.transactions import Transactions  # noqa
from belvo.resources.widget_token import WidgetToken  # noqa



from typing import Any


class BelvoAPIException(Exception):
    ...


class RequestError(BelvoAPIException):

    def __init__(self, status_code: int, detail: Any):
        self.status_code = status_code # HTTP Code
        self.detail = detail # String with a brief description of the error. For detailed information, check our Error documentation on our DevPortal.





from typing import Dict, List, Union

from belvo.resources.base import Resource


class Incomes(Resource):
    endpoint = "/api/incomes/"

    def create(
        self,
        link: str,
        *,
        token: str = None,
        save_data: bool = True,
        date_from: str = None,
        date_to: str = None,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:
        """Retrieve incomes for a link

        Retrieve income insights for checking and savings accounts from a specific link. You can receive insights for a period of up to 365 days, depending on the transaction history available for each bank.

        Example:
        ```python
        # Fetch incomes for a Link
        incomes = client.Incomes.create("b91835f5-6f83-4d9b-a0ad-a5a249f18b7c")

        # Fetch incomes for a Link with and timeout after 15 seconds
        incomes = client.Incomes.create(
            "b91835f5-6f83-4d9b-a0ad-a5a249f18b7c",
            timeout=15
        )

        ```

        Args:
            link (str): The `link.id` that you want to get information for. (UUID).
            token (str, optional): The MFA token generated by the bank in order to access the institution. Defaults to None.
            save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.

        Returns:
            Union[List[Dict], Dict]: For more information on the response from the API, see our [Incomes API documentation](https://developers.belvo.com/reference/retrieveincome).
        """

        data = {"link": link, "save_data": save_data}

        if date_from:
            data.update(date_from=date_from)
        if date_to:
            data.update(date_to=date_to)
        if token:
            data.update(token=token)

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )



from typing import Dict, List, Union

from belvo.resources.base import Resource


class Invoices(Resource):
    endpoint = "/api/invoices/"

    def create(
        self,
        link: str,
        date_from: str,
        date_to: str,
        type_: str,
        *,
        attach_xml: bool = False,
        save_data: bool = True,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:
        """Retrieve invoices for a link

        You can ask for up to **one** year (365 days) of invoices per request. If you need invoices for more than one year, just make another request.

        Example:
        ```python
        # Fetch invoices for a Link
        invoices = client.Invoices.create(
            "b91835f5-6f83-4d9b-a0ad-a5a249f18b7c",
            "2019-07-01",
            "2019-07-31",
            "INFLOW"
        )
        ```

        Args:
            link (str): The fiscal `link.id` that you want to get information for (UUID).
            date_from (str): Date from which you want to start receiving invoices for, in `YYYY-MM-DD` format. The value of `date_from` cannot be greater than `date_to`.
            date_to (str, optional): Date that you want to stop receiving invoirces for, in `YYYY-MM-DD` format. The value of `date_to` cannot be greater than today's date (in other words, no future dates).
            type_ (str): The direction of the invoice (from the perspective of the Link owner). Can be either `OUTFLOW` OR `INFLOW`.
            attach_xml (bool, optional): When set to `True`, you will receive the XML invoice in the response. Defaults to False.
             save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.

        Returns:
            Union[List[Dict], Dict]: For more information on the response from the API, see our [Invoices API documentation](https://developers.belvo.com/reference/retrieveinvoices).
        """

        data = {
            "link": link,
            "date_from": date_from,
            "date_to": date_to,
            "type": type_,
            "attach_xml": attach_xml,
            "save_data": save_data,
        }

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )

    def resume(
        self,
        session: str,
        token: str,
        *,
        link: str = None,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Dict:
        raise NotImplementedError()



from typing import Dict, List, Union

from belvo.resources.base import Resource


class RiskInsights(Resource):
    endpoint = "/api/risk-insights/"

    def create(
        self,
        link: str,
        *,
        token: str = None,
        save_data: bool = True,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:
        """Retrieve risk insights for a link

        Example:
            ```python
            # Fetch risk insights for a Link
            risk_insights = client.RiskInsights.create("44d309dc-24c6-4734-99e0-22c595fee2c2")

            # Fetch risk insights for a Link with and timeout after 15 seconds
            risk_insights = client.RiskInsights.create(
                "44d309dc-24c6-4734-99e0-22c595fee2c2",
                timeout=15
            )

            ```

        Args:
            link (str): The `link.id` that you want to get information for (UUID).
            token (str, optional): The MFA token generated by the bank in order to access the institution. Defaults to None.
            save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.

        Returns:
            Dict: The details of the object. For more information on the response from the API, see our [Risk insights API documentation](https://developers.belvo.com/reference/retrieveriskinsights).
        """

        data = {"link": link, "save_data": save_data}

        if token:
            data.update(token=token)

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )



from datetime import date
from typing import Dict, List, Union

from belvo.resources.base import Resource


class Transactions(Resource):
    endpoint = "/api/transactions/"

    def create(
        self,
        link: str,
        date_from: str,
        *,
        date_to: str = None,
        account: str = None,
        token: str = None,
        save_data: bool = True,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:
        """Retrieve transactions for a link

        <div style="background-color:#f4f6f8; border-left: 6px solid
        #0663F9;padding: 12px;margin-left: 25px; border-radius: 4px; margin-right:
        25px"> <strong>Info: </strong> When retrieving transactions, it is important
        to understand that the available transaction data ranges depend on each
        institution. <br><br>

        If you try to access older information than what we can access, we will
        return all the data we can read within that date range. For example, if you
        request transactions for the last year and we can only access the last six
        months, we will return the information corresponding to these six months of
        data. </div>

        Example:
            ```python
            # Fetch transactions for a Link
            transactions = client.Transactions.create(
                "b91835f5-6f83-4d9b-a0ad-a5a249f18b7c",
                "2019-07-01",
                date_to="2019-07-31"
            )

            # Fetch transactions for a Link with timeout after 15 seconds
            transactions = client.Transactions.create(
                "b91835f5-6f83-4d9b-a0ad-a5a249f18b7c",
                "2019-07-01",
                date_to="2019-07-31",    
                timeout=15
            )

            ```

        Args:
            link (str): The `link.id` that you want to get information for (UUID).
            date_from (str): The date from which you want to start getting transactions for, in `YYYY-MM-DD` format. The value of `date_from` cannot be greater than `date_to`.
            date_to (str, optional): The date you want to stop getting transactions for, in `YYYY-MM-DD` format. The value of `date_to` cannot be greater than today's date (in other words, no future dates). Defaults to today.
            account (str, optional): If provided, we return transactions only from this account (UUID). Defaults to None.
            token (str, optional): The MFA token generated by the bank in order to access the institution. Defaults to None.
            save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.

        Returns:
            Dict: The details of the object. For more information on the response from the API, see our [Transactions API documentation](https://developers.belvo.com/reference/retrievetransactions).
        """

        date_to = date_to or date.today().isoformat()

        data = {"link": link, "date_from": date_from, "date_to": date_to, "save_data": save_data}

        if account:
            data.update(account=account)
        if token:
            data.update(token=token)

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )



from typing import Dict, List, Union

from belvo.resources.base import Resource


class Statements(Resource):
    """
    <br>
    <div style="background-color:#f9c806; border-left: 6px solid #f9c806;padding: 12px;margin-left: 25px; border-radius: 4px; margin-right: 25px">
    <strong>⚠️ Sunset notice: </strong> The Statements resource is due to be sunsetted on 09.05.2022. After which, we will no longer support the resource and we will remove it from our documentation, API, and SDKs. 
    </div>

    """
    endpoint = "/api/statements/"

    def create(
        self,
        link: str,
        account: str,
        year: str,
        month: str,
        *,
        attach_pdf: bool = False,
        save_data: bool = True,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:
        """
        <br>
        <div style="background-color:#f9c806; border-left: 6px solid #f9c806;padding: 12px;margin-left: 25px; border-radius: 4px; margin-right: 25px">
        <strong>⚠️ Sunset notice: </strong> The Statements resource is due to be sunsetted on 09.05.2022. After which, we will no longer support the resource and we will remove it from our documentation, API, and SDKs. 
        </div>

        """

        data = {
            "link": link,
            "account": account,
            "year": year,
            "month": month,
            "attach_pdf": attach_pdf,
            "save_data": save_data,
        }

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )

    def resume(
        self,
        session: str,
        token: str,
        *,
        link: str = None,
        account: str = None,
        raise_exception: bool = False,
        **kwargs,
    ) -> Union[List[Dict], Dict]:

        data = {"session": session, "token": token}

        if link is not None:
            data.update(link=link)

        if account is not None:
            data.update(account=account)

        return self.session.patch(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )



from typing import Any, Dict, Optional

from belvo.resources.base import Resource


class WidgetToken(Resource):
    endpoint = "/api/token/"

    def create(
        self,
        *,
        scopes: Optional[str] = None,
        link: Optional[str] = None,
        widget: Optional[Dict[str, Any]] = None,
        raise_exception: bool = False,
    ):
        """Create a widget access token

        Args:
            scopes (Optional[str], optional): The scope of data to retrieve. We automatically default to`read_institutions, write_links, read_links`.
            link (Optional[str], optional): The `link_id` you want to update credentials for. For more information, see our [Connect Widget in Update Mode](https://developers.belvo.com/docs/connect-widget-update-mode) devportal article. Defaults to None.
            widget (Optional[Dict[str, Any]], optional): Optional object where you can provide custom branding. For more information, see our [Branding and customization](https://developers.belvo.com/docs/widget-branding-and-customization) devportal article. Defaults to None.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.

        Returns:
            Your widget access token.
        """
        if scopes is None:
            scopes = "read_institutions,write_links,read_links"

        data: Dict[str, Any] = {
            "id": self.session._secret_key_id,
            "password": self.session._secret_key_password,
            "scopes": scopes,
        }

        if link:
            data.update(link_id=link)

        if widget:
            data.update(widget=widget)

        return self.session.post(self.endpoint, data=data, raise_exception=raise_exception)



from datetime import date
from typing import Dict, List, Union

from belvo.resources.base import Resource


class Balances(Resource):
    endpoint = "/api/balances/"

    def create(
        self,
        link: str,
        date_from: str,
        *,
        date_to: str = None,
        account: str = None,
        token: str = None,
        save_data: bool = True,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:
        """Retrieve balances for a link

        Retrieve balances from one or more accounts for a specific link within a specified date range. You must provide a Link and a date range defined by `date_from` and `date_to`.

        Example:
        ```python
        # Fetch balances for a Link
        balances = client.Balances.create(
            "b91835f5-6f83-4d9b-a0ad-a5a249f18b7c",
            "2019-07-01",
            date_to="2019-07-31"
        )

        # Fetch balances for a Link with timeout after 15 seconds
        balances = client.Balances.create(
            "b91835f5-6f83-4d9b-a0ad-a5a249f18b7c",
            "2019-07-01",
            date_to="2019-07-31",    
            timeout=15
        )
        ```

        Args:
            link (str): The `link.id` that you want to get information for (UUID).
            date_from (str): Date from which you want to start receiving balances, in `YYYY-MM-DD` format. The value of `date_from` cannot be greater than `date_to`.
            date_to (str, optional): Date that you want to stop receiving balances, in `YYYY-MM-DD` format. The value of `date_to` cannot be greater than today's date (in other words, no future dates). If you do not provide a `date_to` we automatically set the date to today.
            account (str, optional): If provided, only balances matching this `account.id` are returned (UUID). Defaults to None.
            token (str, optional): The MFA token generated by the bank in order to access the institution. Defaults to None.
            save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.

        Returns:
            Union[List[Dict], Dict]: For more information on the response from the API, see our [Balances API documentation](https://developers.belvo.com/reference/retrievebalances).
        """

        date_to = date_to or date.today().isoformat()

        data = {"link": link, "date_from": date_from, "date_to": date_to, "save_data": save_data}

        if account:
            data.update(account=account)
        if token:
            data.update(token=token)

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )



from typing import Dict, Generator, List, Union

from belvo.http import APISession


class Resource:
    endpoint: str

    def __init__(self, session: APISession) -> None:
        self._session = session

    @property
    def session(self) -> APISession:
        return self._session

    def list(self, **kwargs) -> Generator:
        """ # **List all items for the given resource.**

        With the `.list()` method, you can list all items for a given resource. You can additionally add filters to your method in order to only retrieve results matching your query. If you don't provide any filters, we return all items for that resource.

        Example:

        ```
        # Retrieve all accounts (no filter given)
        accounts = client.Accounts.list()

        # Retrieve accounts for a specific bank
        accounts = client.Accounts.list(institution="erebor_mx_retail")


        # Retrieve all checking accounts with an available balance >= 100
        accounts = client.Accounts.list(type__in="checking", balance_available__gte=100)

        ```

        You can find the filters, along with examples, that you can apply per resource by checking our API reference documentation. For example, if you want to see the filters you can apply to the Links resource, then see our [List all links documentation](https://developers.belvo.com/reference/listlinks).

        Returns:
            _type_: _description_

        Yields:
            Generator: _description_
        """
        endpoint = self.endpoint
        return self.session.list(endpoint, params=kwargs)

    def get(self, id: str, **kwargs) -> Dict:
        """#Get the details for a specific object

        Args:
            id (str): The ID of the item you want to get details for (UUID).

        Returns:
            Dict: The details of the object.
        """
        return self.session.get(self.endpoint, id, params=kwargs)
        

    def delete(self, id: str) -> bool:
        """ # Delete an item from the Belvo API.

        Note: If you delete an account object, all associated transactions and owners are also deleted from your Belvo account.

        Example:

        ```
        # Deleting an account
        client.Accounts.delete("161a5e4d-67f5-4760-ae4f-c1fe85cb20ca")

        ```

        Args:
            id (str): The id of the object to delete.

        Returns:
            bool: Returns `True` if the object was deleted.
        """
        return self.session.delete(self.endpoint, id)

    def resume(
        self, session: str, token: str, *, link: str = None, raise_exception: bool = False, **kwargs
    ) -> Union[List[Dict], Dict]:
        """# Resume a resource sessions

        Example:

        ```
        # Login to Belvo API
        client = Client("Secret Key ID", "Secret Key PASSWORD", "sandbox")
        
        # Resume a link
        link = client.Links.resume(
          session="session_id",
          token="token",
          link="link_id"
        )

        ```

        Args:
            session (str): The session you want to resume (UUID).
            token (str): The MFA token required to continue the session.
            link (str, optional): The Link ID . Defaults to None.
            raise_exception (bool, optional): Boolean indicating whether to return an API error or to raise an exception. Defaults to False.

        Returns:
            Union[List[Dict], Dict]: On success, returns the resource data requests.
        """

        data = {"session": session, "token": token}

        if link is not None:
            data.update(link=link)

        return self.session.patch(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )



from typing import Dict, List, Union

from belvo.resources.base import Resource


class Accounts(Resource):
    endpoint = "/api/accounts/"

    def create(
        self,
        link: str,
        *,
        token: str = None,
        save_data: bool = True,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:
        """Retrieve accounts for a given link

        To fetch accounts you will make use of the .create() method and will retrieve all account data available from the banking or gig institution.

        If the account already exists in our records, only its balance and collected_at will be updated.

        > For more information detailed information regarding the Accounts resource, check our [Accounts DevPortal documentation](https://developers.belvo.com/reference/accounts-1).

        Example:

            ```python
            # Fetch accounts for a Link
            accounts = client.Accounts.create("b91835f5-6f83-4d9b-a0ad-a5a249f18b7c")
            ```

        Args:
            link (str): The `link.id` that you want to get information for (UUID).
            token (str, optional): The MFA token generated by the bank in order to access the institution. Defaults to None.
            save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.

        Returns:
            Dict: The details of the object. For more information on the response from the API, see our [Accounts API documentation](https://developers.belvo.com/reference/retrieveaccounts).
        """

        data = {"link": link, "save_data": save_data}

        if token:
            data.update(token=token)

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )



from datetime import date
from typing import Dict, List, Union

from belvo.resources.base import Resource


class InvestmentsTransactions(Resource):
    """
        <br>
        <div style="background-color:#f9c806;padding: 6px; border-radius: 4px">
        <strong>In Development </strong><br>
        This resource is currently in development.
        </div>

    """
    endpoint = "/investments/transactions/"

    def create(
        self,
        link: str,
        date_from: str,
        *,
        date_to: str = None,
        token: str = None,
        save_data: bool = True,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:

        date_to = date_to or date.today().isoformat()

        data = {"link": link, "date_from": date_from, "date_to": date_to, "save_data": save_data}

        if token:
            data.update(token=token)

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )



from typing import Dict, List, Union

from belvo.resources.base import Resource


class TaxComplianceStatus(Resource):
    endpoint = "/api/tax-compliance-status/"

    def create(
        self,
        link: str,
        *,
        attach_pdf: bool = False,
        save_data: bool = True,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:
        """Retrieve a tax compliance status for a link

        Example:
        ```python
        # Fetch tax compliance status for a Link
        tax_compliance_status = client.TaxComplianceStatus.create(
            "b91835f5-6f83-4d9b-a0ad-a5a249f18b7c"
        )

        # Fetch tax compliance status for a Link and retrieve its pdf
        tax_compliance_status = client.TaxComplianceStatus.create(
            "b91835f5-6f83-4d9b-a0ad-a5a249f18b7c",
            attach_pdf=True
        )
        ```

        Args:
            link (str): The fiscal `link.id` that you want to get information for (UUID).
            attach_pdf (bool, optional): When set to `True`, you will receive the PDF in binary format in the response. Defaults to `False`.
            save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.

        Returns:
            Dict: The details of the object. For more information on the response from the API, see our [Tax compliance status API documentation](https://developers.belvo.com/reference/retrievetaxcompliancestatus).
        """

        data = {"link": link, "attach_pdf": attach_pdf, "save_data": save_data}

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )

    def resume(
        self,
        session: str,
        token: str,
        *,
        link: str = None,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Dict:
        raise NotImplementedError()



import logging
from typing import Dict, Generator, List, Union

from requests import HTTPError, Session

from belvo import __version__
from belvo.exceptions import RequestError

logger = logging.getLogger(__name__)


class APISession:
    _secret_key_id: str
    _secret_key_password: str
    _url: str

    def __init__(self, url: str) -> None:
        self._url = url
        self._session = Session()
        self._session.headers.update({"User-Agent": f"belvo-python ({__version__})"})

    @property
    def url(self) -> Union[str, None]:
        return self._url

    @property
    def key_id(self) -> Union[str, None]:
        return self._secret_key_id

    @property
    def session(self) -> Session:
        return self._session

    @property
    def headers(self) -> Dict:
        return self.session.headers  # type: ignore

    def login(self, secret_key_id: str, secret_key_password: str, timeout: int = 5) -> bool:
        base_url = "{}/api/".format(self.url)
        self._secret_key_id = secret_key_id
        self._secret_key_password = secret_key_password
        self._session.auth = (secret_key_id, secret_key_password)

        try:
            r = self.session.get(base_url, timeout=timeout)
            r.raise_for_status()
        except HTTPError:
            return False
        return True

    def _get(self, url: str, params: Dict = None) -> Dict:
        if params is None:
            params = {}

        timeout = params.pop("timeout", 5)
        r = self.session.get(url=url, params=params, timeout=timeout)
        r.raise_for_status()
        return r.json()

    def get(self, endpoint: str, id: str, params: Dict = None) -> Dict:
        url = "{}{}{}/".format(self.url, endpoint, id)
        return self._get(url=url, params=params)

    def put(
        self, endpoint: str, id: str, data: Dict, raise_exception: bool = False, **kwargs
    ) -> Union[List[Dict], Dict]:
        url = "{}{}{}/".format(self.url, endpoint, id)
        r = self.session.put(url=url, json=data, **kwargs)

        if raise_exception:
            try:
                r.raise_for_status()
            except HTTPError:
                raise RequestError(r.status_code, r.json())

        return r.json()

    def list(self, endpoint: str, params: Dict = None) -> Generator:
        url = "{}{}".format(self.url, endpoint)
        while True:
            data = self._get(url, params=params)
            for result in data["results"]:
                yield result

            if not data["next"]:
                break

            url = data["next"]
            params = None

    def post(
        self, endpoint: str, data: Dict, raise_exception: bool = False, *args, **kwargs
    ) -> Union[List, Dict]:
        url = "{}{}".format(self.url, endpoint)
        r = self.session.post(url, json=data, **kwargs)

        if raise_exception:
            try:
                r.raise_for_status()
            except HTTPError:
                raise RequestError(r.status_code, r.json())

        return r.json()

    def patch(
        self, endpoint: str, data: Dict, raise_exception: bool = False, **kwargs
    ) -> Union[List[Dict], Dict]:
        url = "{}{}".format(self.url, endpoint)
        r = self.session.patch(url=url, json=data, **kwargs)

        if raise_exception:
            try:
                r.raise_for_status()
            except HTTPError:
                raise RequestError(r.status_code, r.json())

        return r.json()

    def delete(self, endpoint: str, id: str, timeout: int = 5) -> bool:
        url = "{}{}{}/".format(self.url, endpoint, id)
        r = self.session.delete(url, timeout=timeout)
        try:
            r.raise_for_status()
        except HTTPError:
            return False
        return True



import os

from . import resources
from .enums import Environment
from .exceptions import BelvoAPIException
from .http import APISession


class Client:
    def __init__(self, secret_key_id: str, secret_key_password: str, url: str = None) -> None:
        """# Connect to the Belvo API

        In order to use Belvo API, you will have to login into a new session by using
        a _secret key_.

        Secret keys are generated from the Belvo API dashboard. For more information, please visit 
        [our Developers portal](https://developers.belvo.com/docs/get-your-belvo-api-keys)

        ### Method

        ```python
        def __init__(secret_key_id: str, secret_key_password: str, url: str = None) -> None:
            ...
        ```

        You **must** provide your `secret_key_id` and `secret_key_password`. 

        The `url` tells the client to which Belvo API host should attempt to connect, 
        this allows you to switch from a sandbox to a production environment.

        You can also set which Belvo API host to use, by setting the `BELVO_API_URL`
        environment variable.

        When creating a new instance of `Client`, it will automatically perform a login
        and create a `JWTSession` (if the credentials are valid).
        

        ### Example
        ```python
        # Creating a client instance to connect to Belvo API
        from belvo.client import Client

        my_client = Client(
            "your-secret-key-id", 
            "your-secret-key-password", 
            "https://api.belvo.com"
        )


        # Creating a client that takes url from the environment.
        # We assume that you have set BELVO_API_URL before 
        # (e.g. export BELVO_API_URL=https://sandbox.belvo.com
        my_client = Client(
            "your-secret-key-id", 
            "your-secret-key-password"
        )
        ```

        ## Nested resources

        All resources in the Belvo API are nested attributes in your client instance,
        these resources are available **only** if you provide valid credentials.

        Args:
            secret_key_id (str): Your Belvo secret ID.
            secret_key_password (str): Your Belvo secret password.
            url (str, optional): The URL of the environment you want to connect to.

        Raises:
            BelvoAPIException
        """
        if url is None:
            url = os.getenv("BELVO_API_URL")

        url = Environment.get_url(url)
        if not url:
            raise BelvoAPIException("You need to provide a URL or a valid environment.")

        self.session = APISession(url)

        if not self.session.login(secret_key_id, secret_key_password):
            raise BelvoAPIException("Login failed.")

        self._links = resources.Links(self.session)
        self._accounts = resources.Accounts(self.session)
        self._transactions = resources.Transactions(self.session)
        self._balances = resources.Balances(self.session)
        self._institutions = resources.Institutions(self.session)
        self._incomes = resources.Incomes(self.session)
        self._owners = resources.Owners(self.session)
        self._invoices = resources.Invoices(self.session)
        self._recurring_expenses = resources.RecurringExpenses(self.session)
        self._risk_insights = resources.RiskInsights(self.session)
        self._tax_returns = resources.TaxReturns(self.session)
        self._tax_status = resources.TaxStatus(self.session)
        self._tax_compliance_status = resources.TaxComplianceStatus(self.session)
        self._statements = resources.Statements(self.session)
        self._widget_token = resources.WidgetToken(self.session)
        self._investments_portfolios = resources.InvestmentsPortfolios(self.session)
        self._investments_transactions = resources.InvestmentsTransactions(self.session)

    @property
    def Links(self):
        return self._links

    @property
    def Accounts(self):
        return self._accounts

    @property
    def Transactions(self):
        return self._transactions

    @property
    def Balances(self):
        return self._balances

    @property
    def Institutions(self):
        return self._institutions

    @property
    def Incomes(self):
        return self._incomes

    @property
    def Owners(self):
        return self._owners

    @property
    def Invoices(self):
        return self._invoices

    @property
    def RecurringExpenses(self):
        return self._recurring_expenses

    @property
    def RiskInsights(self):
        return self._risk_insights

    @property
    def TaxReturns(self):
        return self._tax_returns

    @property
    def TaxComplianceStatus(self):
        return self._tax_compliance_status

    @property
    def TaxStatus(self):
        return self._tax_status

    @property
    def Statements(self):
        return self._statements

    @property
    def WidgetToken(self):
        return self._widget_token

    @property
    def InvestmentsPortfolios(self):
        return self._investments_portfolios

    @property
    def InvestmentsTransactions(self):
        return self._investments_transactions



from typing import Dict

from belvo.resources.base import Resource


class Institutions(Resource):
    """
        <br>
        <div style="background-color:#f9c806;padding: 6px; border-radius: 4px">
        <strong>Not implemented</strong><br>
        Although included in the SDK, the delete and resume methods for the Institutions resource will raise a <code>NotImplementedError()</code> when used. 
        </div>

    """
    endpoint = "/api/institutions/"

    def delete(self, id: str) -> bool:
        raise NotImplementedError()

    def resume(
        self,
        session: str,
        token: str,
        *,
        link: str = None,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Dict:
        raise NotImplementedError()



from enum import Enum, unique
from typing import Optional


@unique
class AccessMode(Enum):
    SINGLE = "single"
    RECURRENT = "recurrent"


@unique
class TaxReturnType(Enum):
    YEARLY = "yearly"
    MONTHLY = "monthly"


@unique
class Environment(Enum):
    SANDBOX = "https://sandbox.belvo.com"
    DEVELOPMENT = "https://development.belvo.com"
    PRODUCTION = "https://api.belvo.com"

    @classmethod
    def get_url(cls, environment: Optional[str]):
        if not environment:
            return None

        try:
            return cls[environment.upper()].value
        except KeyError:
            return environment



#!/usr/bin/env python
import os

from setuptools import setup

from belvo import __version__

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

with open(os.path.join(here, "requirements/base.txt")) as f:
    requirements = f.read().splitlines()


def get_packages():
    """
    Return root package and all sub-packages.
    """
    return [
        dirpath
        for dirpath, dirnames, filenames in os.walk("belvo")
        if os.path.exists(os.path.join(dirpath, "__init__.py"))
    ]


setup(
    name="belvo-python",
    url="https://github.com/belvo-finance/belvo-python",
    version=__version__,
    description="Belvo Python SDK",
    python_requires=">=3.6, <4",
    install_requires=requirements,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=get_packages(),
)



from typing import Dict, List, Union

from belvo.resources.base import Resource


class InvestmentsPortfolios(Resource):
    """
    <br>
    <div style="background-color:#bce9af;padding: 6px; border-radius: 4px">
      <strong>🎉 In Beta </strong><br>
      This resource is currently in open beta for all users. If you see any issues, don't hesitate to contact our support team. 
      </div>

    """
    endpoint = "/investments/portfolios/"

    def create(
        self,
        link: str,
        *,
        token: str = None,
        save_data: bool = True,
        raise_exception: bool = False,
        **kwargs: Dict,
    ) -> Union[List[Dict], Dict]:
        """Retrieve portfolios for a link

        Example:
            ```python
            # Fetch investments portfolios for a Link
            investments_portfolios = client.InvestmentsPortfolios.create("b91835f5-6f83-4d9b-a0ad-a5a249f18b7c")
            ```

        Args:
            link (str): The `link.id` that you want to get information for (UUID).
            token (str, optional): The MFA token generated by the bank in order to access the institution. Defaults to None.
            save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.

        Returns:
            Dict: The details of the object. For more information on the response from the API, see our [Investment portfolios API documentation](https://developers.belvo.com/reference/retrieveportfolio).
        """

        data = {"link": link, "save_data": save_data}

        if token:
            data.update(token=token)

        return self.session.post(
            self.endpoint, data=data, raise_exception=raise_exception, **kwargs
        )



import warnings
from typing import Dict, List, Optional, Union

from belvo.enums import AccessMode
from belvo.resources.base import Resource
from belvo.utils import read_file_to_b64


class Links(Resource):
    endpoint = "/api/links/"

    def create(
        self,
        institution: str,
        username: str,
        password: str,
        *,
        username2: str = None,
        username3: str = None,
        password2: str = None,
        token: str = None,
        save_data: bool = True,
        raise_exception: bool = False,
        access_mode: Optional[AccessMode] = None,
        username_type: str = None,
        certificate: str = None,
        private_key: str = None,
        external_id: str = None,
    ) -> Union[List[Dict], Dict]:
        """Register a new Link

        Register a new link with your Belvo account. For in-depth information for all the request parameters, please see our [Links API documentation](https://developers.belvo.com/reference/registerlink).

        <div style="background-color:#f4f6f8; border-left: 6px solid #0663F9;padding: 12px;margin-left: 25px; border-radius: 4px; margin-right: 25px">
        <strong>Info:</strong> We really recommend using our Connect Widget to create links. It'll save you a lot of headaches!.
        </div>


        Args:
            institution (str): The Belvo name for the institution.
            username (str): The end-user's username used to log in to the institution.
            password (str): The end-user's password used to log in to the institution.
            username2 (str, optional): The end-user's second username used to log in to the institution. Defaults to None.
            username3 (str, optional): The end-user's third username used to log in to the institution. Defaults to None.
            password2 (str, optional): The end-user's second password used to log in to the institution. Defaults to None.
            token (str, optional): The MFA token required by the bank to log in. We do not recommend sending the authentication token in the same request as registering the user. See our Handling multi-factor authentication article for more information and best practices. Defaults to None.
            save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.
            access_mode (Optional[AccessMode], optional): The type of link to create. Defaults to None.
            username_type (str, optional): Type of document to be used as a username. Defaults to None.
            certificate (str, optional): For certain fiscal institutions, it is possible to log in using a certificate and a private key, which enables a faster connection to the institution. Defaults to None.
            private_key (str, optional): For certain fiscal institutions, it is possible to log in using a certificate and a private key, which enables a faster connection to the institution. Defaults to None.
            external_id (str, optional): An additional identifier for the link, provided by you, to store in the Belvo database. Defaults to None.

        Returns:
            Union[List[Dict], Dict]: For more information on the response from the API, see our [Links API documentation](https://developers.belvo.com/reference/registerlink).
        """

        data = {
            "institution": institution,
            "username": username,
            "password": password,
            "save_data": save_data,
            "access_mode": access_mode and access_mode.value,
            "username2": username2,
            "username3": username3,
            "password2": password2,
            "token": token,
            "username_type": username_type,
            "certificate": certificate and read_file_to_b64(certificate),
            "private_key": private_key and read_file_to_b64(private_key),
            "external_id": external_id,
        }

        clean_data = {key: value for key, value in data.items() if value}

        return self.session.post(self.endpoint, data=clean_data, raise_exception=raise_exception)

    def update(
        self,
        link: str,
        *,
        password: str = None,
        password2: str = None,
        token: str = None,
        save_data: bool = True,
        raise_exception: bool = False,
        username_type: str = None,
        certificate: str = None,
        private_key: str = None,
    ) -> Union[List[Dict], Dict]:
        """Update a link's credentials

        Update the credentials of a specific link. If the successfully updated link is a recurrent one, we automatically trigger an update of the link. If we find fresh data, you'll receive historical update webhooks.

        Args:
            link (str): The link.id you want to update.
            password (str): The end-user's password used to log in to the institution.
            password2 (str, optional): The end-user's second password used to log in to the institution. Defaults to None.
            token (str, optional): The MFA token required by the bank to log in. We do not recommend sending the authentication token in the same request as registering the user. See our Handling multi-factor authentication article for more information and best practices. Defaults to None.
            save_data (bool, optional): Indicates whether or not to persist the data in Belvo. Defaults to `True`.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.
            username_type (str, optional): Type of document to be used as a username. Defaults to None.
            certificate (str, optional): For certain fiscal institutions, it is possible to log in using a certificate and a private key, which enables a faster connection to the institution. Defaults to None.
            private_key (str, optional): For certain fiscal institutions, it is possible to log in using a certificate and a private key, which enables a faster connection to the institution. Defaults to None.

        Returns:
            Union[List[Dict], Dict]: For more information on the response from the API, see our [Links API documentation](https://developers.belvo.com/reference/patchlinks).
        """


        data = {
            "password": password,
            "save_data": save_data,
            "password2": password2,
            "token": token,
            "username_type": username_type,
            "certificate": certificate and read_file_to_b64(certificate),
            "private_key": private_key and read_file_to_b64(private_key),
        }

        clean_data = {key: value for key, value in data.items() if value}

        return self.session.put(
            self.endpoint, id=link, data=clean_data, raise_exception=raise_exception
        )

    def token(
        self, link: str, scopes: str, *, widget: dict = None, raise_exception: bool = False
    ) -> Union[List[Dict], Dict]:
        from belvo.resources import WidgetToken

        warnings.warn(
            "Please make use of `client.WidgetToken.create(link=<link:uuid>)` "
            "to request a link scoped token instead.",
            DeprecationWarning,
        )

        token = WidgetToken(self.session)
        return token.create(
            scopes=scopes, link=link, widget=widget, raise_exception=raise_exception
        )

    def patch(
        self, link: str, *, access_mode: Optional[AccessMode] = None, raise_exception: bool = False
    ) -> Union[List[Dict], Dict]:
        """Update a link's access_mode

        Args:
            link (str): The `link.id` you want to update the `access_mode` for (UUID).
            access_mode (Optional[AccessMode], optional): The new `access_mode` you want to assign to the link. Can be either `single` or `recurrent`. Defaults to None.
            raise_exception (bool, optional): Indicates whether to raise an exception or return the API error. Defaults to `False`.

        Returns:
            Dict: The details of the object.
        """
        data = {"access_mode": access_mode and access_mode.value}
        return self.session.patch(
            f"{self.endpoint}{link}/", data=data, raise_exception=raise_exception
        )



import pathlib
from base64 import b64encode


def read_file_to_b64(path: str):
    """Read a string in base64-encoded format

    This helper method is used in the SDK when a provided input needs to be base64 encoded (for example, when providing the certificate and private key to create a fiscal link).

    Args:
        path (str): The string to read and convert.

    Raises:
        ValueError: If the provided file is not a string, raises a ValueError.

    Returns:
        The string in base64-encoded format.
    """
    file = pathlib.Path(path)
    if file.exists() and file.is_file():
        with open(path, "rb") as fr:
            return b64encode(fr.read())
    else:
        raise ValueError("Invalid file path")
